%function ID1 is used for the identification of the first order 
%plus dead time model.
%The input of this function is the output-input data y, u, and the time
%series corresponding to the input-output data.
%The time must be evenly spaced.
%Usage:
%[K, theta, tau]=id_tune(y,u,t,dmin)
%dmin is the dead time
%The result is the first order plus dead time parameters: K, theta and tau. 
%i.e. the model will look like 
% 
%K*exp(-theta)
%----------
%1+tau*s

%updated version
%
function [K, theta, tau,yp]=id1(y,u,t,dmin)

dmin=floor(dmin);
if dmin <1
   dmin=1;
end


disp('  Starting to estimate FOPDT constants');
disp(' ');
%get the sampling time

T=t(2)-t(1);
y0=0;
u0=0;

y=y-y0;
u=u-u0;

%get the size of time array

sizet=length(t);

%varify the time evenness
if abs((t(sizet)-t(1))/T -sizet+1) >0.001
   disp('Sampling is not in even time space!')
end


sizeu=size(u);
sizey=size(y);

%varify the size of input and output data

if sizeu ~= sizey
   disp('input and output data must have the same number!')
end


%select the structure of the model --dead time 

%NN=struc(1,1,1:20);

%get the identification matrix
%z=[y,u];
%zv=z;


%get the optimal model
%V=arxstruc(z(1:200,:),zv(201:400,:),NN);
%nn=selstruc(V,0);
sizey=size(y);
if sizey(1)==1
   y=y';
end
sizeu=size(u);
if sizeu(1)==1
   u=u';
end



na=1;
nb=1;
z=[y u];

%e1=0;
%e2=0;
%e3=0;
haha=0;
eee3=[];
%get the dead time and the ARX model parameters
np=1;
se=100000000000;
%dmin=nk;
dmax=dmin*1.5;
%get a best estimate of nk (dead time)
for nk=dmin:4:dmax
nn=[na nb nk];
th=arx(z,nn);
yp=predict(z,th,length(y));
e1=(yp-y)'*(yp-y);
if (e1<se)
   se=e1;
   np=nk;
end

end

%fine selection of dead time nk
if np>=4
for nk=np-3:1:np+3
nn=[na nb nk];
th=arx(z,nn);
yp=predict(z,th,length(y));
e1=(yp-y)'*(yp-y);
if (e1<se)
   se=e1;
   np=nk;
end
end
end

if np<4
for nk=1:1:np+3
nn=[na nb nk];
th=arx(z,nn);
yp=predict(z,th,length(y));
e1=(yp-y)'*(yp-y);
if (e1<se)
   se=e1;
   np=nk;
end
end
end



%get the ARX model
nk=np;
nn=[na nb nk];
th=arx(z,nn);

yp=predict(z,th,length(y));
yp=yp+y0;

%return back to ARX model 
[A,B,dA,dB]=th2arx(th);

%back to the first order plus dead time model

a1=A(2);
b1=B(nk+1);

aa=log(-a1)/(-T);
lk=b1;
d=nk*T;

ck=lk/(1+a1);
ct=1/aa;

K=ck;
Tc=ct;

tau=Tc;
theta=d;
%print out the result
% disp(strcat('gain K is: ',num2str(real(ck))));
% disp ('What are we waiting for?');
%  disp(strcat('time constant tau is: ', num2str(real(ct))));
%  disp(strcat('dead time theta is:  ', num2str(real(d))));
disp(' ');
disp('The first order plus dead time model identified is:');
disp(' ');
disp(' ');

disp(['       ',num2str(real(K))]);
disp(strcat('       ----------*exp(-',num2str(real(d)),'*s)'));
disp(strcat('       1+',num2str(real(Tc)),'*s'));

%tune(K,tau,theta);

%hold off
%eee3
%plot(eee3);